import pytest
from typing import Any
from unittest import mock

from gcs_migration.models import MappedCollection

from gcs_migration.check.mapped_collection import (
    MappedCollectionCheck,
    MC01,
    MC02,
    MC03,
    MC04,
    MC05,
    MC06,
    MC07,
    MC08,
    MC09,
    MC10,
    MC11,
    MC12,
    MC13,
    MC14,
    MC15,
)


def test_check_mc01(mapped_collection: MappedCollection) -> None:
    mc01 = MC01()

    assert not mc01.is_ok(None)
    assert mc01.is_ok(mapped_collection)

    checker = MappedCollectionCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, MC01), res)) != []

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC01), res)) == []


def test_check_mc02(mapped_collection: MappedCollection) -> None:
    mc02 = MC02()

    assert mc02.is_ok(None)
    assert mc02.is_ok(mapped_collection)

    checker = MappedCollectionCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, MC02), res)) == []

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC02), res)) == []

    mapped_collection.display_name = None

    assert not mc02.is_ok(mapped_collection)

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC02), res)) != []


def test_check_mc03(mapped_collection: MappedCollection) -> None:
    mc03 = MC03()
    mapped_collection.force_verify = True

    assert mc03.is_ok(None)
    assert mc03.is_ok(mapped_collection)

    checker = MappedCollectionCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, MC03), res)) == []

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC03), res)) == []

    mapped_collection.force_verify = False
    assert mc03.is_ok(mapped_collection)
    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC03), res)) == []

    mapped_collection.force_verify = None

    assert not mc03.is_ok(mapped_collection)

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC03), res)) != []


def test_check_mc04(mapped_collection: MappedCollection) -> None:
    mc04 = MC04()
    mapped_collection.disable_verify = True

    assert mc04.is_ok(None)
    assert mc04.is_ok(mapped_collection)

    checker = MappedCollectionCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, MC04), res)) == []

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC04), res)) == []

    mapped_collection.disable_verify = False
    assert mc04.is_ok(mapped_collection)
    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC04), res)) == []

    mapped_collection.disable_verify = None

    assert not mc04.is_ok(mapped_collection)

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC04), res)) != []


def test_check_mc05(mapped_collection: MappedCollection) -> None:
    mc05 = MC05()
    mapped_collection.enable_https = True

    assert mc05.is_ok(None)
    assert mc05.is_ok(mapped_collection)

    checker = MappedCollectionCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, MC05), res)) == []

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC05), res)) == []

    mapped_collection.enable_https = False
    assert mc05.is_ok(mapped_collection)
    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC05), res)) == []

    mapped_collection.enable_https = None

    assert not mc05.is_ok(mapped_collection)

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC05), res)) != []


def test_check_mc06(mapped_collection: MappedCollection) -> None:
    mc06 = MC06()
    mapped_collection.allow_guest_collections = True

    assert mc06.is_ok(None)
    assert mc06.is_ok(mapped_collection)

    checker = MappedCollectionCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, MC06), res)) == []

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC06), res)) == []

    mapped_collection.allow_guest_collections = False
    assert mc06.is_ok(mapped_collection)
    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC06), res)) == []

    mapped_collection.allow_guest_collections = None

    assert not mc06.is_ok(mapped_collection)

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC06), res)) != []


@pytest.mark.parametrize(
    "allow_guest_collections,allow_set,expect",
    [
        (True, True, True),
        (True, False, True),
        (False, False, True),
        (False, True, False),
    ],
)
def test_check_mc07(
    mapped_collection: MappedCollection,
    allow_guest_collections: bool,
    allow_set: bool,
    expect: bool,
) -> None:
    mc07 = MC07()
    mapped_collection.allow_guest_collections = allow_guest_collections
    if allow_set:
        mapped_collection.sharing_users_allow = ["user1"]
    else:
        mapped_collection.sharing_users_allow = None

    assert mc07.is_ok(None)
    assert mc07.is_ok(mapped_collection) == expect

    checker = MappedCollectionCheck()
    res = checker.check(mapped_collection)
    if expect:
        assert list(filter(lambda x: isinstance(x, MC07), res)) == []
    else:
        assert list(filter(lambda x: isinstance(x, MC07), res)) != []


@pytest.mark.parametrize(
    "allow_guest_collections,deny_set,expect",
    [
        (True, True, True),
        (True, False, True),
        (False, False, True),
        (False, True, False),
    ],
)
def test_check_mc08(
    mapped_collection: MappedCollection,
    allow_guest_collections: bool,
    deny_set: bool,
    expect: bool,
) -> None:
    mc08 = MC08()
    mapped_collection.allow_guest_collections = allow_guest_collections
    if deny_set:
        mapped_collection.sharing_users_deny = ["user1"]
    else:
        mapped_collection.sharing_users_deny = None

    assert mc08.is_ok(None)
    assert mc08.is_ok(mapped_collection) == expect

    checker = MappedCollectionCheck()
    res = checker.check(mapped_collection)
    if expect:
        assert list(filter(lambda x: isinstance(x, MC08), res)) == []
    else:
        assert list(filter(lambda x: isinstance(x, MC08), res)) != []


@pytest.mark.parametrize(
    "allow_guest_collections,allow_set,expect",
    [
        (True, True, True),
        (True, False, True),
        (False, False, True),
        (False, True, False),
    ],
)
def test_check_mc09(
    mapped_collection: MappedCollection,
    allow_guest_collections: bool,
    allow_set: bool,
    expect: bool,
) -> None:
    mc09 = MC09()
    mapped_collection.allow_guest_collections = allow_guest_collections
    if allow_set:
        mapped_collection.sharing_groups_allow = ["user1"]
    else:
        mapped_collection.sharing_groups_allow = None

    assert mc09.is_ok(None)
    assert mc09.is_ok(mapped_collection) == expect

    checker = MappedCollectionCheck()
    res = checker.check(mapped_collection)
    if expect:
        assert list(filter(lambda x: isinstance(x, MC09), res)) == []
    else:
        assert list(filter(lambda x: isinstance(x, MC09), res)) != []


@pytest.mark.parametrize(
    "allow_guest_collections,deny_set,expect",
    [
        (True, True, True),
        (True, False, True),
        (False, False, True),
        (False, True, False),
    ],
)
def test_check_mc10(
    mapped_collection: MappedCollection,
    allow_guest_collections: bool,
    deny_set: bool,
    expect: bool,
) -> None:
    mc10 = MC10()
    mapped_collection.allow_guest_collections = allow_guest_collections
    if deny_set:
        mapped_collection.sharing_groups_deny = ["user1"]
    else:
        mapped_collection.sharing_groups_deny = None

    assert mc10.is_ok(None)
    assert mc10.is_ok(mapped_collection) == expect

    checker = MappedCollectionCheck()
    res = checker.check(mapped_collection)
    if expect:
        assert list(filter(lambda x: isinstance(x, MC10), res)) == []
    else:
        assert list(filter(lambda x: isinstance(x, MC10), res)) != []


@pytest.mark.parametrize(
    "name,value,expect",
    [
        (
            "sharing_dn",
            "/C=US/O=Globus Consortium/OU=Globus Online/OU=Transfer User/CN=__transfer__",
            True,
        ),
        ("sharing_dn", "/C=US/O=Other/CN=Sharing", False),
    ],
)
def test_check_mc11(
    mapped_collection: MappedCollection, name: str, value: str, expect: bool,
) -> None:
    mc11 = MC11()
    assert mapped_collection.gridftp_config_data is not None
    mapped_collection.gridftp_config_data[name] = value

    assert mc11.is_ok(None)
    assert mc11.is_ok(mapped_collection) == expect

    checker = MappedCollectionCheck()
    res = checker.check(mapped_collection)
    if expect:
        assert list(filter(lambda x: isinstance(x, MC11), res)) == []
    else:
        assert list(filter(lambda x: isinstance(x, MC11), res)) != []


@pytest.mark.parametrize(
    "name,value,expect",
    [("load_dsi_module", "file", True), ("load_dsi_module", "blackpearl", False)],
)
def test_check_mc12(
    mapped_collection: MappedCollection, name: str, value: str, expect: bool,
) -> None:
    mc12 = MC12()
    assert mapped_collection.gridftp_config_data is not None
    mapped_collection.gridftp_config_data[name] = value

    assert mc12.is_ok(None)
    assert mc12.is_ok(mapped_collection) == expect

    checker = MappedCollectionCheck()
    res = checker.check(mapped_collection)
    if expect:
        assert list(filter(lambda x: isinstance(x, MC12), res)) == []
    else:
        assert list(filter(lambda x: isinstance(x, MC12), res)) != []


@pytest.mark.parametrize(
    "name, value",
    [
        ("banner", "banner message"),
        ("banner_file", "/etc/banner"),
        ("banner_terse", True),
        ("login_msg", "message"),
        ("login_msg_file", "/etc/message"),
        ("control_interface", "192.168.1.2"),
        ("data_interface", "192.168.1.2"),
        ("port_range", "1,1000"),
        ("$GLOBUS_TCP_SOURCE_RANGE", "1,1000"),
        ("hostname", "hostname.example.org"),
    ],
)
def test_check_mc13(mapped_collection: MappedCollection, name: str, value: Any) -> None:
    mc13 = MC13()
    assert mapped_collection.gridftp_config_data is not None
    mapped_collection.gridftp_config_data[name] = value

    assert mc13.is_ok(None)
    with mock.patch("gcs_migration.check.mapped_collection.print") as p:
        assert mc13.is_ok(mapped_collection)
        output = "\n".join([line[1][0] for line in p.mock_calls])
        assert name in output

    checker = MappedCollectionCheck()
    with mock.patch("gcs_migration.check.mapped_collection.print") as p:
        res = checker.check(mapped_collection)
        assert list(filter(lambda x: isinstance(x, MC13), res)) == []
        output = "\n".join([line[1][0] for line in p.mock_calls])
        assert name in output


@pytest.mark.parametrize(
    "name, value",
    [
        ("xnetmgr", "netman"),
        ("log_module", "log_module"),
        ("log_unique", "/log/unique"),
        ("log_transfer", "/log/transfer"),
        ("log_filemode", "0600"),
        ("exec", "/sbin/globus-gridftp-server"),
        ("fork", False),
        ("fork_fallback", True),
        ("single", True),
        ("debug", True),
        ("ignore_bad_threads", True),
        ("bad_signal_exit", False),
        ("blocksize", 64),
        ("sync_writes", True),
        ("direct", True),
        ("perms", "0700"),
        ("file_timeout", 120),
        ("slow_dirlist", 120),
        ("checksum_throttle", 120),
        ("allow_udt", True),
        ("chdir", False),
        ("chdir_to", "/gridftp"),
        ("allow_root", True),
        ("allow_disabled_login", True),
        ("connections_max", 100),
        ("control_preauth_timeout", 99),
        ("control_idle_timeout", 99),
        ("connections_disabled", True),
        ("offline_msg", "stop"),
    ],
)
def test_check_mc14(mapped_collection: MappedCollection, name: str, value: Any) -> None:
    mc14 = MC14()
    assert mapped_collection.gridftp_config_data is not None
    mapped_collection.gridftp_config_data[name] = value

    assert mc14.is_ok(None)
    with mock.patch("gcs_migration.check.mapped_collection.print") as p:
        assert mc14.is_ok(mapped_collection)
        output = "\n".join([line[1][0] for line in p.mock_calls])
        assert name in output

    checker = MappedCollectionCheck()
    with mock.patch("gcs_migration.check.mapped_collection.print") as p:
        res = checker.check(mapped_collection)
        assert list(filter(lambda x: isinstance(x, MC14), res)) == []
        output = "\n".join([line[1][0] for line in p.mock_calls])
        assert name in output


@pytest.mark.parametrize(
    "name, value",
    [
        ("ssh", True),
        ("chroot_path", "/chroot"),
        ("ipc_interface", "192.168.1.1"),
        ("ipc_allow_from", "192.168.1.0/24"),
        ("ipc_deny_from", "192.168.1.0/24"),
        ("secure_ipc", False),
        ("ipc_auth_mode", "self"),
        ("ipc_user_name", "user"),
        ("ipc_subject", "/CN=IPC"),
        ("ipc_credential", "/etc/ipc.pk12"),
        ("ipc_cookie", "yummy"),
        ("ipc_port", 123),
        ("ipc_idle_timeout", 99),
        ("ipc_connect_timeout", 99),
        ("remote_nodes", "192.168.1.2"),
        ("hybrid", True),
        ("data_node", True),
        ("stripe_blocksize", 1024),
        ("stripe_count", 2),
        ("brain", "zombie"),
        ("stripe_layout", 3),
        ("stripe_blocksize_locked", True),
        ("stripe_layout_locked", True),
        ("stripe_mode", 2),
        ("allow_anonymous", True),
        ("anonymous_names_allowed", "user"),
        ("anonymous_user", "user"),
        ("anonymous_group", "group"),
        ("pw_file", "/etc/passwd"),
        ("sharing_control", False),
        ("allowed_modules", "file"),
        ("dc_whitelist", "tcp"),
        ("fs_whitelist", "file"),
        ("popen_whitelist", "popen"),
        ("dc_default", "tcp"),
        ("fs_default", "file"),
        ("cas", False),
        ("acl", "aclmod"),
        ("auth_level", 0),
        ("process_user", "user"),
        ("process_group", "group"),
        ("threads", 42),
        ("inetd", True),
        ("daemon", False),
        ("detach", True),
        ("pidfile", "/var/pidfile"),
        ("use_home_dirs", False),
        ("home_dir", "/heart"),
        ("rp_follow_symlinks", True),
        ("disable_command_list", "mlst"),
        ("globus_location", "/opt/globus"),
    ],
)
def test_check_mc15(mapped_collection: MappedCollection, name: str, value: Any) -> None:
    mc15 = MC15()
    assert mapped_collection.gridftp_config_data is not None
    mapped_collection.gridftp_config_data[name] = value

    assert mc15.is_ok(None)
    with mock.patch("gcs_migration.check.mapped_collection.print") as p:
        assert mc15.is_ok(mapped_collection)
        output = "\n".join([line[1][0] for line in p.mock_calls])
        assert name in output

    checker = MappedCollectionCheck()
    with mock.patch("gcs_migration.check.mapped_collection.print") as p:
        res = checker.check(mapped_collection)
        assert list(filter(lambda x: isinstance(x, MC15), res)) == []
        output = "\n".join([line[1][0] for line in p.mock_calls])
        assert name in output
