from typing import Any, Generator, List, Optional, Tuple
from unittest import mock

import pytest
from click.testing import CliRunner

from gcs_migration.commands.check import check
from gcs_migration.models import MigrationPlan


@pytest.fixture
def mock_check_environment(
    mock_get_transfer_api: Any,
    mock_get_auth_api: Any,
    migration_plan_cmd_create: MigrationPlan,
    authorization_response_success: Tuple[Optional[str], str, int],
) -> Generator[MigrationPlan, None, None]:
    with mock.patch.object(MigrationPlan, "load") as mock_migration_load:
        mock_migration_load.return_value = migration_plan_cmd_create
        yield migration_plan_cmd_create


@pytest.mark.parametrize(
    "args,expect",
    [
        ([], 4),
        (["--type-check", "acl"], 0),
        (["--type-check", "credential"], 2),
        (["--type-check", "guest-collection"], 0),
        (["--type-check", "mapped-collection"], 0),
        (["--type-check", "migration-plan"], 1),
        (["--type-check", "role"], 0),
        (["--type-check", "storage-gateway"], 1),
        (["--type-check", "storage-gateway", "--type-check", "credential"], 3),
    ],
    ids=[
        "all",
        "acl",
        "credential",
        "guest-collection",
        "mapped-collection",
        "migration-plan",
        "role",
        "storage-gateway",
        "storage-gateway-and-credential",
    ],
)
def test_check(
    mock_check_environment: MigrationPlan, args: List[str], expect: int,
) -> None:
    runner = CliRunner()
    result = runner.invoke(check, args, catch_exceptions=False)
    assert result.exit_code == expect


def test_requires_logged_in(mock_not_logged_in: None) -> None:
    runner = CliRunner()
    result = runner.invoke(check, catch_exceptions=False)
    assert "you will have to login" in result.output
