from typing import Any, Generator, Optional, Tuple
from unittest import mock

import pytest
from pytest_lazyfixture import lazy_fixture

from click.testing import CliRunner

from gcs_migration.commands.disable_guest_collection import disable_guest_collection
from gcs_migration.models import MigrationPlan


@pytest.fixture
def mock_migration_plan(
    mock_get_transfer_api: Any,
    mock_get_auth_api: Any,
    migration_plan_cmd_create: MigrationPlan,
    authorization_response_success: Tuple[Optional[str], str, int],
) -> Generator[MigrationPlan, None, None]:
    with mock.patch.object(MigrationPlan, "load") as mock_migration_load:
        mock_migration_load.return_value = migration_plan_cmd_create
        yield migration_plan_cmd_create


@pytest.mark.parametrize(
    "guest_collection_id, expect",
    [
        (lazy_fixture("share_endpoint_uuid"), 0),
        ("40e88cad-3a51-4262-951c-9d29e8eba729", 1),
    ],
)
def test_check(
    mock_migration_plan: MigrationPlan, guest_collection_id: str, expect: int,
) -> None:
    runner = CliRunner()
    result = runner.invoke(disable_guest_collection, [guest_collection_id])
    assert result.exit_code == expect
