from typing import List

import pytest

from gcs_migration.models import ACL, Credential, CredentialDict, GuestCollection
from gcs_migration.typing.transfer_documents import (
    AccessListDocumentDict,
    AnonymousEffectivePermissionsDict,
    EndpointDocumentDict,
)


@pytest.fixture
def guest_collection(
    shared_endpoint_document: EndpointDocumentDict,
    access_list_document: AccessListDocumentDict,
    anonymous_effective_permissions_document: AnonymousEffectivePermissionsDict,
    share_state_file: str,
    credential: CredentialDict,
) -> GuestCollection:
    guest_collection = GuestCollection()
    guest_collection.transfer_endpoint = shared_endpoint_document
    guest_collection.access_list = access_list_document
    guest_collection.anonymous_effective_permissions = (
        anonymous_effective_permissions_document
    )
    guest_collection.share_state_file = share_state_file

    cred = Credential(credential)
    guest_collection.credential = cred

    return guest_collection


@pytest.fixture
def guest_collection_unresolved_credential(
    shared_endpoint_document: EndpointDocumentDict,
    access_list_document: AccessListDocumentDict,
    anonymous_effective_permissions_document: AnonymousEffectivePermissionsDict,
    share_state_file: str,
    credential_unresolved: CredentialDict,
) -> GuestCollection:
    guest_collection = GuestCollection()
    guest_collection.transfer_endpoint = shared_endpoint_document
    guest_collection.access_list = access_list_document
    guest_collection.anonymous_effective_permissions = (
        anonymous_effective_permissions_document
    )
    guest_collection.share_state_file = share_state_file

    cred = Credential(credential_unresolved)
    guest_collection.credential = cred

    return guest_collection


@pytest.fixture
def new_acls() -> List[ACL]:
    return [
        ACL(
            data={
                "id": id,
                "principal": principal,
                "principal_type": "identity",
                "path": "/some-new-acl-path",
                "permissions": "rw",
                "create_time": f"2021-04-06T19:{id % 60}:08+00:00",
            }
        )
        for (id, principal) in [
            (42, "6024d5b9-d843-47d7-8c98-4c3673cb43e3"),
            (314, "6c200665-e635-40bf-8726-53602ed26d56"),
        ]
    ]
